import Utils from './../../API/IKUtils';
//
export default class IKDBBaseExpression  {
  constructor() {
    this.values = [];
    this.operator = IKDBExpressionOperator.AND;
    this.counter = 0;
  }
  /*public methods*/
  appendOperator(key, val, cond) {
    const aKey = this._anonimizeKeyValue(key);
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `${keyName} ${cond} ${aKey}`, values: [val], appendKeys: [aKey],
                      anonimizedNames: { [keyName]: key }});
  }
  appendBetween(key, val1, val2) {
    const aKey = this._anonimizeKeyValue(key);
    const aKey2 = this._anonimizeKeyValue(key);
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `(${keyName} BETWEEN ${aKey} AND ${aKey2})`, values: [val1, val2], appendKeys: [aKey, aKey2],
                      anonimizedNames: { [keyName]: key }});
  }
  appendBeginsWith(key, val) {
    const aKey = this._anonimizeKeyValue(key);
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `begins_with(${keyName}, ${aKey})`, values: [val], appendKeys: [aKey],
                      anonimizedNames: { [keyName]: key }});
  }
  appendAttributeType(key, val) {
    const aKey = this._anonimizeKeyValue(key);
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `attribute_type(${keyName}, ${aKey})`, values: [val], appendKeys: [aKey],
                      anonimizedNames: { [keyName]: key }});
  }
  appendContains(key, val) {
    const aKey = this._anonimizeKeyValue(key);
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `contains(${keyName}, ${aKey})`, values: [val], appendKeys: [aKey],
                      anonimizedNames: { [keyName]: key }});
  }
  appendExists(key) {
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `attribute_exists(${keyName})`, anonimizedNames: { [keyName]: key } });
  }
  appendNotExists(key) {
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `attribute_not_exists(${keyName})`, anonimizedNames: { [keyName]: key } });
  }
  appendIN(key, values) {
    let aKeys = values.map((val)=>this._anonimizeKeyValue(val));
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `(${keyName} IN (${aKeys.toString()}))`, values: values, appendKeys: aKeys,
                      anonimizedNames: { [keyName]: key }});
  }
  appendNotIN(key, values) {
    let aKeys = values.map((val)=>this._anonimizeKeyValue(val));
    const keyName = this._anonimizeKeyName(key);
    this.values.push({exp: `NOT (${keyName} IN (${aKeys.toString()}))`, values: values, appendKeys: aKeys,
                      anonimizedNames: { [keyName]: key }});
  }
  _anonimizeKeyValue(_key) {
    this.counter++;
    return (':' + _key.replace(/[:-]/g, '') + this.counter);
  }
  _anonimizeKeyName(_key) {
    this.counter++;
    return ('#' + _key.replace(/[:-]/g, '') + this.counter);
  }
  //Groups
  startGroupping() { this.values.push({startGroupping: true}); }
  endGroupping() { this.values.push({endGroupping: true}); }
  addSeparator(val) { this.values.push({exp: ` ${val} `}); }
  //Builds items and exp (if set)
  buildExpression() {
    const exp = this.__buildExpressionOnQuery();
    // if (exp && Object.keys(exp).length > 0) console.debug('Building query expression..', exp);
    return exp;
  }
  __buildExpressionOnQuery() {
    let expression = {};
    if (this.values.length > 0) {
      //Check required values
      if (!expression.items) expression.items = {};
      if (!expression.names) expression.names = {};
      //build conditional expression
      let condition = '';
      let groupping = false;
      let keyCount = 0;
      for (let addObj of this.values) {
        if (condition != '' && !groupping) condition += ` ${this.operator} `;
        //Check for include types
        if (addObj.startGroupping) {
          condition += `(`;
          groupping = true;
        } else if (addObj.endGroupping) {
          condition += `)`;
          groupping = false;
        } else if (addObj.values) { //multi value expressions
          condition += `${addObj.exp}`;
          let i = 0;
          for (let val of addObj.values) {
            expression.items[`${addObj.appendKeys[i]}`] = Utils.encapsulateForDB(val);
            i++;
          }
        } else if (addObj.exp) { //pure expressions
          condition += `${addObj.exp}`;
        }
        //Check for key anonimization
        if (addObj.anonimizedNames) {
          for (let aKeyName of Object.keys(addObj.anonimizedNames)) {
            expression.names[aKeyName] = addObj.anonimizedNames[aKeyName];
          }
        }
      } expression.exp = condition;
    } return expression;
  }
}
//
export const IKDBExpressionOperator = {
  EQUAL: '=',
  NOT_EQUAL: '<>',
  LESS_THAN: '<',
  LESS_EQUAL_THAN: '<=',
  GREATER_THAN: '>',
  GREATER_EQUAL_THAN: '>=',
  GREATE_EQUAL_THAN: '>=', //typo compatiblity :/ 
  NOT: 'NOT',
  AND: 'AND',
  OR: 'OR'
};
