import IKDBBaseExpression from './IKDBBaseExpression';
import Utils from './../../API/IKUtils';
//
import IKDBBaseQuery from './../Prototype/IKDBBaseQuery';
//
export default class IKDBDDBBaseQuery extends IKDBBaseQuery {
  constructor(optIndexName) {
    super();
    this.keyValues = {};
    this.filter = new IKDBBaseExpression();
    this.keyConditional = new IKDBBaseExpression();
    this.conditional = new IKDBBaseExpression();
    this.isConsistentRead = false;
    this.indexName = '';
    this.exclusiveStartKey = null;
    //Compatiblity
    this.ExclusiveStartKey = null;
    if (optIndexName) this.indexName = optIndexName;
  }
  //Key
  appendKey(value, keyName, prefix) { this.keyValues[keyName] = (prefix != undefined && value.indexOf(prefix) != 0 ? prefix + value : value); }

  //Inheritance support
  _rawQuery(dbTableName, isTransaction) {
    return {
      ...this._buildBaseQuery(),
      ...(isTransaction ? {} : {ReturnConsumedCapacity:'TOTAL'}),
      ...(this.indexName != '' ? {IndexName: this.indexName} : {}), //
      ...(this.isConsistentRead ? {ConsistentRead: true} : {}), //
      ...(dbTableName ? {TableName: dbTableName} : {}) //table is options
    };
  }

  /*private methods*/
  //Builds ExpressionAttributeValues and ConditionExpression (if set)
  _buildBaseQuery() {
    //console.debug('Building base query..');
    let expression = {}; //init query
    //ConditionExpression
    expression = this.__buildConditionalExpression(expression);
    //ConditionKeyExpression
    expression = this.__buildConditionalKeyExpression(expression);
    //FilterExpression
    expression = this.__buildFilterExpression(expression);
    //Key expression
    expression = this.__buildKeysExpression(expression);
    //Exclusive start key
    expression = this.__buildExclusiveStartKey(expression);
    return expression;
  }
  __buildKeysExpression(expression) {
    if (Object.keys(this.keyValues).length > 0) {
      if (!expression.Key) expression.Key = {}; //create over demand
      for (let dbKey of Object.keys(this.keyValues)) {
        expression.Key[dbKey] = Utils.encapsulateForDB(this.keyValues[dbKey]);
      }
    } return expression;
  }
  __buildConditionalExpression(expression) {
    let expObj = this.conditional.buildExpression();
    //Check items and exp separately because they are not dependant
    if (expObj.items && Object.keys(expObj.items).length > 0) {
      //merge objects - priority to existing keys
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      expression.ExpressionAttributeValues = Object.assign(expression.ExpressionAttributeValues, expObj.items);
    }
    //Check names and exp separately because they are not dependant
    if (expObj.names && Object.keys(expObj.names).length > 0) {
      //merge objects - priority to existing keys
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      expression.ExpressionAttributeNames = Object.assign(expression.ExpressionAttributeNames, expObj.names);
    }
    if (expObj.exp && expObj.exp.length > 0) expression.ConditionExpression = expObj.exp;
    //
    return expression;
  }
  __buildConditionalKeyExpression(expression) {
    let expObj = this.keyConditional.buildExpression();
    //Check items and exp separately because they are not dependant
    if (expObj.items && Object.keys(expObj.items).length > 0) {
      //merge objects - priorityu to existing keys
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      expression.ExpressionAttributeValues = Object.assign(expression.ExpressionAttributeValues, expObj.items);
    }
    //Check names and exp separately because they are not dependant
    if (expObj.names && Object.keys(expObj.names).length > 0) {
      //merge objects - priority to existing keys
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      expression.ExpressionAttributeNames = Object.assign(expression.ExpressionAttributeNames, expObj.names);
    }
    if (expObj.exp && expObj.exp.length > 0) expression.KeyConditionExpression = expObj.exp;
    //
    return expression;
  }
  __buildFilterExpression(expression) {
    let expObj = this.filter.buildExpression();
    //Check items and exp separately because they are not dependant
    if (expObj.items && Object.keys(expObj.items).length > 0) {
      //merge objects - priorityu to existing keys
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      expression.ExpressionAttributeValues = Object.assign(expression.ExpressionAttributeValues, expObj.items);
    }
    //Check names and exp separately because they are not dependant
    if (expObj.names && Object.keys(expObj.names).length > 0) {
      //merge objects - priority to existing keys
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      expression.ExpressionAttributeNames = Object.assign(expression.ExpressionAttributeNames, expObj.names);
    }
    if (expObj.exp && expObj.exp.length > 0) expression.FilterExpression = expObj.exp;
    //
    return expression;
  }
  __buildExclusiveStartKey(expression) {
    if (this.ExclusiveStartKey) {
      expression.ExclusiveStartKey = this.ExclusiveStartKey;
    } else if (this.exclusiveStartKey) {
      expression.ExclusiveStartKey = this.exclusiveStartKey;
    }
    return expression;
  }
}
