import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryScan extends IKDBBaseQuery {
  constructor(optIndexName) {
    super(optIndexName);
    this.limit = -1;
    this.lastEvaluatedKey = null;
  }
  async run(dbManager, appendingItems) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const resp = await this.rawRun(dbManager, appendingItems);
    if (resp.Items) {
      let unmarshalled = Utils.decapsulateForDB(resp.Items);
      localConsole.debug('Parsed result: ', unmarshalled);
      //Check for appending items from previous query
      if (appendingItems && appendingItems.length > 0) {
        unmarshalled = unmarshalled.concat(appendingItems);
      }
      //Check for continuation
      this.lastEvaluatedKey = resp.LastEvaluatedKey;
      if (resp.LastEvaluatedKey && !(this.limit > 0 && unmarshalled.length >= this.limit)) {
        localConsole.log('Using continuation key:', resp.LastEvaluatedKey, unmarshalled.length, this.limit);
        this.exclusiveStartKey = resp.LastEvaluatedKey;
        return this.run(dbManager, unmarshalled);
      } return unmarshalled;
    } return null;
  }
  async rawRun(dbManager, appendingItems) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const query = {
      ...this._rawQuery(dbManager.tableName),
      ...(this.limit > 0 ? { Limit: this.limit - (appendingItems ? appendingItems.length : 0) } : {})
    };
    localConsole.log('Scanning: ', query, appendingItems ? appendingItems.length : 0);
    const resp = await dbManager.connection.scan(query);
    // localConsole.debug('Raw result: ', resp);
    this._incrementTrackerStats(resp, dbManager);
    return resp;
  }
  /* tracker support */
  _incrementTrackerStats(resp, dbManager) {
    if (resp && resp.ConsumedCapacity && resp.ConsumedCapacity.CapacityUnits) dbManager.stats.readUnits += parseFloat(resp.ConsumedCapacity.CapacityUnits);
  }
}
