import IKDBBaseQuery from './IKDBBaseQuery';
//
export default class IKDBQueryTransactionalWrite extends IKDBBaseQuery {
  constructor() {
    super();
    this.updateItems = [];
    this.putItems = [];
    this.deleteItems = [];
  }
  /*compatibility*/
  setUpdateItems(items) { this.updateItems = items; }
  setPutItems(items) { this.putItems = items; }
  setDeleteItems(items) { this.deleteItems = items; }
  //
  appendPutQuery(query) { this.putItems.push(query); }
  appendUpdateQuery(query) { this.updateItems.push(query); }
  appendDeleteQuery(query) { this.deleteItems.push(query); }

  async run(dbManager) {
    const resp = await this.rawRun(dbManager);
    return (resp && resp.UnprocessedItems && Object.keys(resp.UnprocessedItems).length == 0); //evaluate success response
  }
  async rawRun(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const query = this._rawQuery(dbManager.tableName);
    localConsole.log('Batch writting items: ', query);
    const resp = await dbManager.connection.transactWriteItems(query);
    // localConsole.debug('Raw result: ', resp);
    return resp;
  }

  /*private methods*/
  _rawQuery(dbTableName) {
    return this._buildBatchWriteQuery(
      { ...super._rawQuery(null /*no table name, since this operation is multitable*/)}, dbTableName);
  }

  //Builds RequestItems
  _buildBatchWriteQuery(expression, dbTableName) {
    console.debug('Building transaction write query..');
    //Check for required keys
    if (!expression.TransactItems) expression.TransactItems = [];
    //Put ops
    expression = this.__buildPutItems(expression, dbTableName);
    //Update ops
    expression = this.__buildUpdateItems(expression, dbTableName);
    //Delete ops
    expression = this.__buildDeleteItems(expression, dbTableName);
    return expression;
  }
  __buildUpdateItems(expression, dbTableName) {
    for (let item of this.putItems.slice(0,24)) {
      if (item instanceof IKDBBaseQuery) item = item._rawQuery(dbTableName, true);
      expression.TransactItems.push({Update: {... item}});
    } return expression;
  }
  __buildPutItems(expression, dbTableName) {
    for (let item of this.putItems.slice(0,24)) {
      if (item instanceof IKDBBaseQuery) item = item._rawQuery(dbTableName, true);
      expression.TransactItems.push({Put: {... item}});
    } return expression;
  }
  __buildDeleteItems(expression, dbTableName) {
    for (let item of this.deleteItems.splice(0, (24 - this.putItems.length))) {
      if (item instanceof IKDBBaseQuery) item = item._rawQuery(dbTableName, true);
      expression.TransactItems.push({Delete: {... item}});
    } return expression;
  }
}
